/*
 * Copyright (c) 2010, 2014, James Bailie.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *     * The name of James Bailie may not be used to endorse or promote
 * products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <fcntl.h>

#include "dombey.h"

void scgi_init_func()
{
   scgi_set_name( "test" );
}

void scgi_exit_func()
{
}

void scgi_worker_init_func()
{
}

void send_headers( FILE *file )
{
   fputs( "Content-Type: text/html; charset=utf-8\n\n"

      "<!DOCTYPE html>\n"
      "<html>\n"
      "<head>\n"

      "<title>Dombey Test Program</title>\n"
      "\n"

      "<meta name=\"description\" content=\"The Dombey 'hello world!'\" />\n"
      "<meta name=\"language\" content=\"EN\" />\n"
      "<meta name=\"keywords\" content=\"SCGI,CGI,application server\" />\n"

      "\n"

      "<style type=\"text/css\">\n"
      "   BODY { margin: 0;\n"
      "          padding: 1%;\n"
      "          color: #404040;\n"
      "          background: #ffffff;\n"
      "   }\n"
      "   TABLE { width: 100% }\n"
      "   TD { padding-left: 1% }\n"
      "</style>\n"

      "\n"

      "</head>\n"
      "<body>\n",

      file
   );
}

void scgi_request_handler( FILE *file )
{
   char **ptr, **env, **params, **files, **cookies, *fname;

   env = scgi_get_envs();
   params = scgi_get_params();
   files = scgi_get_files();
   cookies = scgi_get_cookies();

   send_headers( file );

   fprintf( file,
            "<h1>LibDombey Test Program</h1>\n\n"
            "<p>value of scgi_config_file option: %s</p>\n"
            "<h2>SCGI Environment:</h2>\n\n"
            "<table>",
            ( scgi_config_file == NULL ? "(not set)" : scgi_config_file ));

   for( ptr = env; *ptr != NULL; ptr += 2 )
      fprintf( file, "<tr><td>%s:</td><td>%s</td></tr>\n", *ptr, *( ptr + 1 ));

   fputs( "</table>\n\n<h2>SCGI Parameters:</h2>\n\n<table>", file );

   for( ptr = params; *ptr != NULL; ptr += 2 )
      fprintf( file, "<tr><td>%s:</td><td>%s</td></tr>\n", *ptr, *( ptr + 1 ));

   fputs( "</table>\n\n<h2>Uploaded Files:</h2>\n\n<table>", file );

   for( ptr = files; *ptr != NULL; ptr += 5 )
   {
      fname = scgi_html_escape( *( ptr + 1 ));

      fprintf( file, "<tr><td>%s</td><td>%s</td><td>%s</td></tr>\n", *ptr, fname, *( ptr + 2 ));

      free( fname );
   }

   fputs( "</table>\n\n<h2>Cookies:</h2>\n\n<table>", file );

   for( ptr = cookies; *ptr != NULL; ptr += 2 )
      fprintf( file, "<tr><td>%s:</td><td>%s</td></tr>\n", *ptr, *( ptr + 1 ));

   fputs( "</table>\n</body>\n</html>", file );
}
